package {
	
	import com.flowingdata.gps.MarkersClip;
	import com.modestmaps.Map;
	import com.modestmaps.TweenMap;
	import com.modestmaps.core.MapExtent;
	import com.modestmaps.geo.Location;
	import com.modestmaps.mapproviders.OpenStreetMapProvider;
	import com.modestmaps.mapproviders.microsoft.*;
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.ColorMatrixFilter;
	import flash.geom.ColorTransform;
	import flash.net.*;
	import flash.text.TextField;
	import flash.text.TextFormat;
		
	[SWF(width="900", height="450", backgroundColor="#ffffff", frameRate="32")]
	public class Openings extends Sprite
	{		
		private var stageWidth:Number = 900;
		private var stageHeight:Number = 450;
		
		private var map:Map;
		private var mapWidth:Number = stageWidth;
		private var mapHeight:Number = stageHeight;
		private var markers:MarkersClip;
		private var navButtons:Sprite;
		
		private var urlLoader:URLLoader;
		private var locations:Array = new Array();
		private var openingDates:Array = new Array();
		
		private var yearLabel:TextField;
		private var storeCount:TextField;
		private var textFormat:TextFormat;
		private var metaFormat:TextFormat;
		
		private var startPage:Sprite = new Sprite();
		private var endPage:Sprite = new Sprite();
		private var playButton:Sprite;
		
		
		public function Openings()
		{	
			stage.scaleMode = StageScaleMode.NO_SCALE;
	        stage.align = StageAlign.TOP_LEFT;
			
			// Initialize map			
			map = new TweenMap(mapWidth, mapHeight, true, new OpenStreetMapProvider());
			map.setExtent(new MapExtent(50.259381, 24.324408, -128.320313, -59.941406));
			
			// Grayscale and invert map
			var mat:Array = [0.24688,0.48752,0.0656,0,44.7,0.24688,0.48752,0.0656,0,44.7,0.24688,0.48752,0.0656,0,44.7,0,0,0,1,0];
	        var colorMat:ColorMatrixFilter = new ColorMatrixFilter(mat);
	        map.grid.filters = [colorMat];
	        map.grid.transform.colorTransform = new ColorTransform(-1,-1,-1,1,255,255,255,0);
	        addChild(map);
	        map.x = 0; map.y = 0;
			
			// Load Walmart data
			var urlRequest:URLRequest = new URLRequest('http://projects.flowingdata.com/walmart/walmarts_new.xml');
			urlLoader = new URLLoader();
			urlLoader.addEventListener(Event.COMPLETE, onLoadLocations);
			urlLoader.load(urlRequest);			
			
			// Draw navigation buttons
			drawNavigation();
			
			// Draw Tickers
//			drawTickers();
			
			// Draw start page
//			drawStartPage();
		
		}
		
		
		private function play():void {
			
//			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			markers.startAnimation();
		}
		
		
		private function onLoadLocations(e:Event):void
		{
			var xml:XML = new XML(e.target.data);			
			for each(var w:* in xml.walmart) 
	    	{	
	    		locations.push(new Location(w.latitude, w.longitude));
	    		openingDates.push(String(w.opening_date));
	    	}
	    	
	    	var brightGreen:uint = 0x7CFC00;
	    	markers = new MarkersClip(map, locations, openingDates);
	    	map.addChild(markers);
	    	
	    	play();
		}
	    
	    
	    private function onEnterFrame(e:Event):void
	    {
			yearLabel.text = String(markers.currentYear-1);
	    	yearLabel.setTextFormat(textFormat);
	    
	    	storeCount.text = String(markers.markerIndex);
	    	storeCount.setTextFormat(textFormat);
	    }
		
		
	    
	    // Draw navigation buttons
	    private function drawNavigation():void
	    {
	    	// Navigation buttons (zooming)
			var buttons:Array = new Array();
	        navButtons = new Sprite();
	        addChild(navButtons);
	        buttons.push(makeButton(navButtons, 'plus', '+', map.zoomIn));
	        buttons.push(makeButton(navButtons, 'minus', '–', map.zoomOut));
	        var nextX:Number = 0;
			for(var i:Number = 0; i < buttons.length; i++) {
				var currButton:Sprite = buttons[i];
				Sprite(buttons[i]).scaleX = 3;
				Sprite(buttons[i]).scaleY = 3;
				Sprite(buttons[i]).x = nextX;
				nextX += 3*Sprite(buttons[i]).getChildByName('label').width;
			}
			navButtons.x = 2; navButtons.y = map.height-navButtons.height-2;
	    }
	    
	    
	    
	    
	    // Draw Tickers
	    private function drawTickers():void
	    {
	    	// Tickers
			metaFormat = new TextFormat('Arial', 11, 0xFFFFFF, false);
			var currYearLabel:TextField = new TextField();
			currYearLabel.text = 'YEAR:';
			currYearLabel.setTextFormat(metaFormat);
			addChild(currYearLabel);
			currYearLabel.x = stageWidth-100; currYearLabel.y = stageHeight-130;
			
			yearLabel = new TextField();
			textFormat = new TextFormat('Arial', 22, 0xFFFFFF, true);
			yearLabel.text = '--';
			yearLabel.setTextFormat(textFormat);
			addChild(yearLabel);
			yearLabel.x = currYearLabel.x; yearLabel.y = currYearLabel.y+20;
			
			var storeCountLabel:TextField = new TextField();
			storeCountLabel.text = "STORE COUNT:";
			storeCountLabel.setTextFormat(metaFormat);
			addChild(storeCountLabel);
			storeCountLabel.x = currYearLabel.x; storeCountLabel.y = yearLabel.y + 50;
			
			storeCount = new TextField();
			storeCount.text = '0';
			storeCount.setTextFormat(textFormat);
			addChild(storeCount);
			storeCount.x = yearLabel.x; storeCount.y = yearLabel.y + 70;
	    }
	    
	    
	    private function drawStartPage():void {
	    	// Start page
			startPage.graphics.beginFill(0x000000, 0.6);
			startPage.graphics.drawRect(0, 0, stageWidth, stageHeight);
			addChild(startPage);
			startPage.x = startPage.y = 0;
			
			var titleField:TextField = new TextField();
			var titleFormat:TextFormat = new TextFormat('Arial', 20, 0xFFFFFF, true);
			titleField.text = 'GROWTH OF WALMART, 1962-2006 \nby FlowingData';
			titleField.setTextFormat(titleFormat);
			titleField.width = titleField.textWidth + 10;
			startPage.addChild(titleField);
			titleField.x = 10; titleField.y = 10;
			
			
			// Play button
			playButton = new Sprite();
			playButton.graphics.beginFill(0x7CFC00);
			playButton.graphics.drawRect(0, 0, 60, 30);
			startPage.addChild(playButton);
			playButton.x = stageWidth/2 - 30; playButton.y = stageHeight/2-15;
			
			var playField:TextField = new TextField();
			playField.scaleX = 0.8; playField.scaleY = 0.8;
			playField.text = 'PLAY';
			playField.setTextFormat(titleFormat);
			playField.width = playField.textWidth + 10; playField.height = playField.textHeight + 10;
			playField.selectable = false;
			playButton.addChild(playField);
			playField.x = 6; playField.y = 5;
			playButton.useHandCursor = true;
			playButton.mouseChildren = false;
			playButton.buttonMode = true;
			playButton.addEventListener(MouseEvent.CLICK, 
				function(e:Event):void { 
					removeChild(startPage);
					play(); 
				});
	    }
	    
	    
	    public function makeButton(clip:Sprite, name:String, labelText:String, action:Function):Sprite
	    {
	        var button:Sprite = new Sprite();
	        button.name = name;
	        clip.addChild(button);
	        
	        var label:TextField = new TextField();
	        label.name = 'label';
	        label.selectable = false;
	        label.textColor = 0xffffff;
	        label.text = labelText;
	        label.width = label.textWidth + 4;
	        label.height = label.textHeight + 3;
	        button.addChild(label);
	        
	        button.graphics.moveTo(0, 0);
	        button.graphics.beginFill(0xFDBB30, 1);
	        button.graphics.drawRect(0, 0, label.width, label.height);
	        button.graphics.endFill();

			button.addEventListener(MouseEvent.CLICK, action);
			button.useHandCursor = true;
			button.mouseChildren = false;
			button.buttonMode = true;
	        
	        return button;
	    }
	    
		
	}
}
