package com.flowingdata.gps
{
	import com.modestmaps.Map;
	import com.modestmaps.events.MapEvent;
	
	import flash.display.Sprite;
	import flash.events.TimerEvent;
	import flash.geom.Point;
//	import flash.text.TextField;
//	import flash.text.TextFormat;
	import flash.utils.Timer;
	

	public class MarkersClip extends Sprite
	{
		protected var map:Map;							// Base map
		public var markers:Array;						// Holder for markers
		public var isStationary:Boolean;
		
		public var locations:Array;
		private var openingDates:Array;
		
		private var storesPerYear:Array = new Array();
		private var spyIndex:Number = 0;				// Stores per year index
		private var currentYearCount:Number = 0;		// Stores shown so far in current year
		private var currentRate:Number;					// Number of stores to show 
		private var totalTime:Number = 90000;			// Animation should last approx 1.5 minute
		private var timePerYear:Number;
		public var currentYear:Number = 1962;			// Start with initial year
				
		private var xpoints:Array = new Array();		// Transformed longitude
		private var ypoints:Array = new Array();		// Transformed latitude
		
		public var markerIndex:Number = 0;
		private var starting:Point;
		
		private var pause:Boolean = false;
        public var scaleZoom:Boolean = false;
		
		
		public function MarkersClip(map:Map, locations:Array, openingDates:Array) 
		{	
			this.map = map;
			
			this.x = map.getWidth() / 2;
			this.y = map.getHeight() / 2;
			
			this.locations = locations;
			setPoints();
			setMarkers();
			
			this.openingDates = openingDates;
			
			var tempIndex:int = 0;
			
			storesPerYear = [1,0,1,1,0,2,5,5,5,15,17,19,25,19,27,39,34,43,54,150,63,87,99,110,121,142,125,131,178,163,138,156,107,129,53,60,66,80,105,106,114,96,130,118,37];
			timePerYear = totalTime / storesPerYear.length;			
			
			this.map.addEventListener(MapEvent.START_ZOOMING, onMapStartZooming);
	        this.map.addEventListener(MapEvent.STOP_ZOOMING, onMapStopZooming);
	        this.map.addEventListener(MapEvent.ZOOMED_BY, onMapZoomedBy);
			this.map.addEventListener(MapEvent.START_PANNING, onMapStartPanning);
	        this.map.addEventListener(MapEvent.STOP_PANNING, onMapStopPanning);
	        this.map.addEventListener(MapEvent.PANNED, onMapPanned);
		}
		
		
//		public function play():void {
//			storesPerYearIndex = 0;
//			currentYearCount = 0;
//			markerIndex = 0;
//			resetMarkers();
//			this.addEventListener(Event.ENTER_FRAME, onEnterFrame);
//		} 
		
		
		
		// Place markers on map without displaying them
		protected function setMarkers():void
		{
			markers = new Array();
			
			for (var i:int = 0; i < locations.length; i++) 
			{
				var marker:Marker = new Marker();
				addChild(marker);
				marker.x = xpoints[i]; marker.y = ypoints[i];
				markers.push(marker);
			}
			
		}
		
		
		protected function resetMarkers():void {
			for (var i:int = 0; i < markers.length; i++) {
				Marker(markers[i]).reset();
			}
		}
		
		
		// Calculate x/y coordinates for points based on lat/lon
		public function setPoints():void {
			if (locations == null) {
				return;
			}			
			
			var p:Point;
			for (var i:int = 0; i < locations.length; i++) {			
				p = map.locationPoint(locations[i], this);
				xpoints[i] = p.x;
				ypoints[i] = p.y;
			}			
		}
		
		
		public function updatePoints():void 
		{	
			setPoints();
			
			for (var i:int = 0; i < markers.length; i++) {
				Marker(markers[i]).x = xpoints[i]; Marker(markers[i]).y = ypoints[i];
			}
			
		}
		
		
		public function destroy():void 
		{	
			map.removeEventListener(MapEvent.START_ZOOMING, onMapStartZooming);
	        map.removeEventListener(MapEvent.STOP_ZOOMING, onMapStopZooming);
	        map.removeEventListener(MapEvent.ZOOMED_BY, onMapZoomedBy);
			map.removeEventListener(MapEvent.START_PANNING, onMapStartPanning);
	        map.removeEventListener(MapEvent.STOP_PANNING, onMapStopPanning);
	        map.removeEventListener(MapEvent.PANNED, onMapPanned);
	        map.removeEventListener(MapEvent.EXTENT_CHANGED, onExtentChanged);
	        map.removeEventListener(MapEvent.RESIZED, onMapResized);
		}
		
		
		
		public function startAnimation():void
		{	
			var yearlyTimer:Timer = new Timer(timePerYear, storesPerYear.length);
			yearlyTimer.addEventListener(TimerEvent.TIMER, onNextYear);
			yearlyTimer.start();
		}
		
		
		private function onNextYear(e:TimerEvent):void
		{
			if (storesPerYear[spyIndex] > 0)
			{
				var timePerStore:Number = timePerYear / storesPerYear[spyIndex];
				if (timePerStore == timePerYear) {
					timePerStore = 0;
				}
				var storeTimer:Timer = new Timer(timePerStore, storesPerYear[spyIndex]);
				storeTimer.addEventListener(TimerEvent.TIMER, playNextStore);
				storeTimer.start();
			}
			currentYear++;
			spyIndex++;
		}
		
		
		private function playNextStore(e:TimerEvent):void
		{
			Marker(markers[markerIndex]).play();
			markerIndex++;
		}
		
		
		
		
		/* @ Listeners */
		
		
		private function onMapStartZooming(event:MapEvent):void
	    {
	    	pause = true;
	        for (var i:int = 0; i < markers.length; i++) {
	        	Marker(markers[i]).clear();
	        }
	    }
	    
	    private function onMapStopZooming(event:MapEvent):void
	    {
	    	pause = false;
	        if (scaleZoom) {
	            scaleX = scaleY = 1.0;
	        }
            updatePoints();
            for (var i:int = 0; i < markers.length; i++) {
	        		Marker(markers[i]).draw();
	        }
	    }
	    
	    private function onMapZoomedBy(event:MapEvent):void
	    {
	        if (scaleZoom) {
    	        scaleX = scaleY = Math.pow(2, event.zoomDelta);
	        }
	        else {
                
	        }
	    }
		
		
		private function onMapStartPanning(event:MapEvent):void
	    {
	    	pause = true;
	        starting = new Point(x, y);
	        trace('xpoint on start ' + xpoints[0]);
	    }
		
		
		private function onMapPanned(event:MapEvent):void
	    {
	        if (starting) {
	            x = starting.x + event.panDelta.x;
	            y = starting.y + event.panDelta.y;
	        }
	        else {
	            x = event.panDelta.x;
	            y = event.panDelta.y;	            
	        }
	    }
	    
	    
	    private function onMapStopPanning(event:MapEvent):void
	    {
			pause = false;
	    }
	    
	    private function onExtentChanged(event:MapEvent):void
		{
			updatePoints();
		}
	    
	    
	    private function onMapResized(event:MapEvent):void
	    {
	        x = event.newSize[0]/2;
	        y = event.newSize[1]/2;
	        updatePoints();
	    }
		
		
	}
}